'
' Xorshift RNGs by George Marsaglia
'
' A xorshift random number generator (xorshift RNG) produces a
' sequence of 232-1 integers x, or a sequence of 264-1
' pairs x,y, or a sequence of 296-1 triples x,y,z, etc., by means
' of repeated use of a simple computer construction:
' exclusive-or (xor) a computer word with a shifted version of itself.

' Combining such xorshift operations for various shifts and arguments
' provides extremely fast and simple RNGs that seem to do very well on
' tests of randomness. To give an idea of the power and effectiveness
' of xorshift operations, here is the essential part of a C procedure
' that, with only three xorshift operations per call, will provide 2^128-1
' random 32-bit integers, given four random seeds x,y,z,w:
'
' tmp=(x(x<<15)); x=y; y=z; z=w; return w=(w(w>>21))(tmp(tmp>>4))

' Demo written by Vic Drastik
'
IMPORT  "xst"
IMPORT  "xsx"
CONSOLE

DECLARE FUNCTION VOID  Entry()
DECLARE FUNCTION ULONG XorShift()
'
FUNCTION VOID Entry()
	ULONG u

	FOR i = 1 TO 99
		u = XorShift()
		PRINT i, HEX$(u,8)
	NEXT i
	INLINE$ ("Press any key to quit")

END FUNCTION
'
FUNCTION ULONG XorShift()

	STATIC ULONG x, y, z, w, t, initialised
	XLONG year, month, day, weekday, hour, minute, second, nsec

	' seed x, y, z, w
	IFZ initialised THEN
		initialised = 1
		XstGetDateAndTime(@year, @month, @day, @weekDay, @hour, @minute, @second, @nsec)
		centiseconds$$ = (nsec\10000000)+100*(second+60*(minute+60*(hour+24*(day+31*(month+12*GIANT(year))))))
		x = GLOW(centiseconds$$)
		y = ROTATEL(x,17)
		z = 1 +  GHIGH(centiseconds$$) ' to guarantee state vector is non-zero
		w = ROTATEL(x, 23)
	END IF

	' generate RN
	t = x ^ (x << 15)
	x = y
	y = z
	z = w
	w = (w ^ (w >> 21)) ^ (t ^ (t >> 4))
	RETURN w

END FUNCTION
END PROGRAM
