
' LCDPanel control to mimic 5x7 dot matrix character display
' based on PB code by Jules Marchildon and Chris Boss

' v0.0002
' - modified character array to match ASCII layout, no longer need to use GetCharacterBitmap().
' - modified WM_PAINT routine and BuildPanel() so all of control is painted at once (no flickering)

VERSION "0.0002"
PROGRAM	"lcdpanel"

IMPORT "xst"
IMPORT "xsx"
IMPORT "gdi32"
IMPORT "user32"
IMPORT "kernel32"

EXPORT

$$DISPLAYPANELCLASSNAME = "displaypanel"

$$GWL_PDIDATA = 0             ' controls private instance data
$$MAX_COL     = 40            ' maximum number of colums per line
$$MAX_ROW     = 8             ' maximum number of rows per panel

' custom messages for this control class!
$$LCD_SETSIZE        =   1124   ' wParam = rows, lParam  = cols
$$LCD_SETCOLORSET    =   1125   ' wParam = asciiz pointer to string with color set name
$$LCD_SETMARGINS     =   1126  	' wParam = right margin, lParam = bottom margin
$$LCD_SETBWP         =   1127  	' wParam = border width of panel
$$LCD_SETGAP         =   1128   ' wParam = gap
$$LCD_REBUILD        =   1129   ' wParam = if 1 then resize control for correct borders)
$$LCD_GETPANELHEIGHT =   1130
$$LCD_GETPANELWIDTH  =   1131

END EXPORT

' Character map data...    ' contains the 5x7 character map data
TYPE CHARMAP                 '
  STRING * 1 .chars     ' contains string character ID
  UBYTE .charset[6]     ' contains raw bit map of character
END TYPE

TYPE CHARSET            ' make sure cm() is large enough !!!!
  CHARMAP .cm[223]      ' use @pdi.cc, character count as
END TYPE                ' max when looping through array

' text message buffer
TYPE TXTMSG
	STRING * 40 .msg[7]
END TYPE

TYPE CLRDATA
	STRING*32 .clr
	XLONG .data[8]
END TYPE

' specifications of character panel
TYPE PANELINFO
	XLONG .hPanelDC 			' Panels memory DC to blit to screen DC
	XLONG .nRows  				' number of rows, set a sensible limit = %MAX_ROW
	XLONG .nCols  				' number of columns, set a sensible limit = %MAX_COL
	XLONG .charset  			' specify predefined character set (from picklist)
	STRING*32 .clrset  		' specify predefined color set (from picklist)
	XLONG .colorset[2]  	' Panels colorset table
	XLONG .bkcolor  			' panels frame background color, need for extra painting.
	XLONG .rightMargin  	' Right margin, border
	XLONG .bottomMargin 	' Bottom margin, border
	XLONG .gap  					' Spacing between dots vertically & horizontally
	XLONG .panelX 				' X Dimensions of Panel, width, X=nRight
	XLONG .panelY  				' Y Dimensions of panel, height, Y=nBottom
	XLONG .dotX  					' X dimensions of cell width (internal)
	XLONG .dotY  					' Y dimensions of cell height (internal)
	XLONG .nRight  				' main panels width extent, provided by caller
	XLONG .nBottom 				' main panels height extent, seeded internally
	XLONG .bwp  					' border witdh of panel in pixels
	CHARSET .cs 					' charset, allow enough buffer area
	XLONG .cc 						' character count
	TXTMSG .txt 					' text message buffer
	XLONG .initWidth			' initialized control width
	XLONG .initHeight			' initialized control height
END TYPE

EXPORT
DECLARE FUNCTION LCDPanel ()
DECLARE FUNCTION ConfigureDisplayPanel (hPanel, row, col, colorset$, rmargin, bmargin, bwidth, gap)
DECLARE FUNCTION CreateDisplayPanel (hParent, msg$, ctrlID, x, y, cx, cy, bShow)
END EXPORT

DECLARE FUNCTION Load5x7CharacterMap (hWnd)
DECLARE FUNCTION LCDSetText (hWnd, lpText)
DECLARE FUNCTION GetBorderWidth (hWnd)
DECLARE FUNCTION Create5x7CellDC (hWnd, char)
DECLARE FUNCTION CalculateBestfit (hPanel)
DECLARE FUNCTION GetColorSet (hPanel)
DECLARE FUNCTION BuildPanel (hPanel, mode)
DECLARE FUNCTION RegisterPanelEx ()
DECLARE FUNCTION WndPanelProc (hWnd, msg, wParam, lParam)

FUNCTION LCDPanel ()

 RegisterPanelEx ()

END FUNCTION


FUNCTION Load5x7CharacterMap (hWnd)
' initialize 5x7 character map data into memory
' Returns: character count

	PANELINFO pi
	CHARMAP cmap[]

	ppi = GetWindowLongA (hWnd, $$GWL_PDIDATA)
	IF ppi THEN XstCopyMemory (ppi, &pi, SIZE(pi))

	GOSUB InitFontData
	upp = UBOUND(cmap[])

	FOR i = 0 TO upp
		pi.cs.cm[i].chars = cmap[i].chars
		pi.cs.cm[i].charset[0] = cmap[i].charset[0]
		pi.cs.cm[i].charset[1] = cmap[i].charset[1]
		pi.cs.cm[i].charset[2] = cmap[i].charset[2]
		pi.cs.cm[i].charset[3] = cmap[i].charset[3]
		pi.cs.cm[i].charset[4] = cmap[i].charset[4]
		pi.cs.cm[i].charset[5] = cmap[i].charset[5]
		pi.cs.cm[i].charset[6] = cmap[i].charset[6]
	NEXT i

	pi.cc = upp
	IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))

	RETURN upp ' return total character count

SUB InitFontData

' Char bitmap image of 5x7 font data

input$ = " ,0,0,0,0,0,0,0, _
!,32,32,32,32,0,32,32, _
\",80,80,80,0,0,0,0, _
#,80,80,248,80,248,80,80, _
$,32,120,160,248,40,240,32, _
%,232,168,240,32,120,168,184, _
&,64,160,160,64,168,144,232, _
\x27,16,16,16,0,0,0,0, _
(,16,32,32,32,32,32,16, _
),64,32,32,32,32,32,64, _
*,32,168,112,248,112,168,32, _
+,0,32,32,248,32,32,0, _
\xFF,0,0,0,48,48,16,32, _
-,0,0,0,248,0,0,0, _
.,0,0,0,0,0,96,96, _
/,8,8,16,32,64,128,128, _
0,112,136,152,168,200,136,112, _
1,32,96,32,32,32,32,112, _
2,112,136,8,16,32,64,248, _
3,112,136,8,48,8,136,112, _
4,16,48,80,144,248,16,16, _
5,248,128,128,240,8,8,240, _
6,120,128,128,240,136,136,112, _
7,248,136,8,16,32,32,32, _
8,112,136,136,112,136,136,112, _
9,112,136,136,120,8,136,112, _
:,0,96,96,0,96,96,0, _
;,96,96,0,96,96,32,64, _
<,16,32,64,128,64,32,16, _
=,0,0,248,0,248,0,0, _
>,128,64,32,16,32,64,128, _
?,112,136,8,16,32,0,32, _
@,112,136,184,184,176,128,120, _
A,112,136,136,136,248,136,136, _
B,240,136,136,240,136,136,240, _
C,112,136,128,128,128,136,112, _
D,240,136,136,136,136,136,240, _
E,248,128,128,224,128,128,248, _
F,248,128,128,224,128,128,128, _
G,112,136,128,128,152,136,112, _
H,136,136,136,248,136,136,136, _
I,112,32,32,32,32,32,112, _
J,8,8,8,8,136,136,112, _
K,136,144,160,192,160,144,136, _
L,128,128,128,128,128,128,248, _
M,136,216,168,136,136,136,136, _
N,137,200,168,152,136,136,136, _
O,112,136,136,136,136,136,112, _
P,240,136,136,240,128,128,128, _
Q,112,136,136,136,168,144,104, _
R,240,136,136,240,160,144,136, _
S,120,128,128,112,8,8,240, _
T,248,32,32,32,32,32,32, _
U,136,136,136,136,136,136,112, _
V,136,136,136,80,80,32,32, _
W,136,136,136,168,168,80,80, _
X,136,136,80,32,80,136,136, _
Y,136,136,80,32,32,32,32, _
Z,248,8,16,32,64,128,248, _
[,56,32,32,32,32,32,56, _
\\,128,128,64,32,16,8,8, _
],224,32,32,32,32,32,224, _
^,32,80,136,0,0,0,0, _
_,0,0,0,0,0,0,248, _
`,128,64,32,0,0,0,0, _
a,0,0,112,8,120,136,120, _
b,128,128,240,136,136,136,240, _
c,0,0,120,128,128,128,120, _
d,8,8,120,136,136,136,120, _
e,0,0,112,136,248,128,120, _
f,48,72,64,240,64,64,64, _
g,0,120,136,120,8,136,112, _
h,128,128,128,240,136,136,136, _
i,0,32,0,32,32,32,32, _
j,32,0,32,32,32,160,64, _
k,128,128,144,224,160,144,136, _
l,96,32,32,32,32,32,112, _
m,0,0,208,168,168,168,168, _
n,0,0,176,200,136,136,136, _
o,0,0,112,136,136,136,112, _
p,0,0,240,136,240,128,128, _
q,0,0,120,136,120,8,8, _
r,0,0,176,200,128,128,128, _
s,0,0,120,128,112,8,240, _
t,64,64,240,64,64,72,48, _
u,0,0,136,136,136,152,104, _
v,0,0,136,136,80,80,32, _
w,0,0,136,168,168,80,80, _
x,0,0,136,80,32,80,136, _
y,0,0,136,136,120,8,112, _
z,0,0,248,16,32,64,252, _
\xFE,24,32,32,64,32,32,24, _
|,32,32,32,32,32,32,32, _
\xFD,192,32,32,16,32,32,192, _
~,64,168,16,0,0,0,0"

	DIM cmap[0]
	template$ = "TYPE,STRING*1,UBYTE,UBYTE,UBYTE,UBYTE,UBYTE,UBYTE,UBYTE"
	ret = XstLoadCompositeDataArray (@cmap[], input$, template$, @errornum)

END SUB

END FUNCTION

FUNCTION LCDSetText (hWnd, lpText)

	' set text to panelinfo struct
	' return number of rows of text

	$CRLF = "\r\n"

	PANELINFO pi

	ppi = GetWindowLongA (hWnd, $$GWL_PDIDATA)
	XstCopyMemory (ppi, &pi, SIZE(pi))

	' clear current buffer with spaces
	FOR i = 1 TO $$MAX_ROW
		pi.txt.msg[i-1] = SPACE$($$MAX_COL)
	NEXT i

	text$ = CSTRING$(lpText)
	XstStringToStringArray (@text$, @text$[])
	ct = UBOUND(text$[]) + 1
	IF ct > $$MAX_ROW THEN ct = $$MAX_ROW

	FOR i = 1 TO ct
		t$ = text$[i-1]
		t$ = LEFT$(t$ + SPACE$($$MAX_COL), $$MAX_COL)
		pi.txt.msg[i-1] = t$
	NEXT i

	XstCopyMemory (&pi, ppi, SIZE(pi))

	RETURN ct

END FUNCTION

FUNCTION GetBorderWidth (hWnd)

	ews = GetWindowLongA (hWnd, $$GWL_EXSTYLE)
	IF (ews && $$WS_EX_CLIENTEDGE) <> 0    THEN x = x + GetSystemMetrics($$SM_CXEDGE)
	IF (ews && $$WS_EX_DLGMODALFRAME) <> 0 THEN x = x + GetSystemMetrics($$SM_CXDLGFRAME)
	RETURN x

END FUNCTION


FUNCTION WndPanelProc (hWnd, msg, wParam, lParam)

' DISPLAYPANEL Class callback procedure

	PANELINFO pi
	PAINTSTRUCT ps
	RECT rc
	CREATESTRUCT cs

	ppi = GetWindowLongA (hWnd, $$GWL_PDIDATA)
	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	SELECT CASE msg

		CASE $$LCD_SETSIZE
			IF wParam >= 1 && wParam <= $$MAX_ROW THEN pi.nRows = wParam
			IF lParam >= 1 && lParam <= $$MAX_COL THEN pi.nCols = lParam
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			SendMessageA (hWnd, $$LCD_REBUILD, 1, 0)
			RETURN (1)

		CASE $$LCD_SETCOLORSET
			pi.clrset = CSTRING$(wParam)
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			SendMessageA (hWnd, $$LCD_REBUILD, 0, 0)
			RETURN (1)

		CASE $$LCD_SETMARGINS
			IF wParam >= 1 THEN pi.rightMargin = wParam
			IF lParam >= 1 THEN pi.bottomMargin = lParam
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			SendMessageA (hWnd, $$LCD_REBUILD, 1, 0)
			RETURN (1)

		CASE $$LCD_SETBWP
			IF wParam >= 0 THEN pi.bwp = wParam
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			SendMessageA (hWnd, $$LCD_REBUILD, 1, 0)
			RETURN (1)

		CASE $$LCD_SETGAP
			IF wParam >= 0 THEN pi.gap = wParam
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			SendMessageA (hWnd, $$LCD_REBUILD, 1, 0)
			RETURN (1)

		CASE $$WM_SETTEXT
			LCDSetText (hWnd, lParam)
			SendMessageA (hWnd, $$LCD_REBUILD, 0, 0)
			RETURN (1)

		CASE $$LCD_REBUILD
			IF (pi.hPanelDC) THEN DeleteDC (pi.hPanelDC)
			IF wParam <> 0 THEN
				x = BuildPanel (hWnd, 1)
				IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))
				SetWindowPos (hWnd, NULL, 0, 0, pi.nRight + (pi.bwp*2) + (GetBorderWidth(hWnd)*2), _
				pi.nBottom + (pi.bwp*2) + (GetBorderWidth(hWnd)*2), _
				$$SWP_NOMOVE | $$SWP_NOZORDER)
			ELSE
				x = BuildPanel (hWnd, 0)
				InvalidateRect (hWnd, NULL, 0)
			END IF
			RETURN (1)

		CASE $$WM_CREATE

			' get createstruct
			XstCopyMemory (lParam, &cs, SIZE(cs))

			' allocate memory for panel data
			hHeap = GetProcessHeap ()
			ppi = HeapAlloc (hHeap, $$HEAP_ZERO_MEMORY, SIZE(pi))
			SetWindowLongA (hWnd, $$GWL_PDIDATA, ppi)

			' set lcd text
			rows = LCDSetText (hWnd, cs.lpszName)

			ppi = GetWindowLongA (hWnd, $$GWL_PDIDATA )
			IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

			' set defaults now
			pi.nRows 				= rows
			pi.nCols 				= 40
			pi.clrset				="LCD"
			pi.rightMargin 	= 4
			pi.bottomMargin = 4
			pi.bwp 					= 8
			pi.gap 					= 1
			pi.initWidth    = cs.cx
			pi.initHeight   = cs.cy
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))

			' intialize memory with character map...
			lrt = Load5x7CharacterMap (hWnd)

			' draw panel
			SendMessageA (hWnd, $$LCD_REBUILD, 1, 0)

		CASE $$WM_ERASEBKGND
			RETURN (1)

		CASE $$WM_PAINT
			hDC = BeginPaint (hWnd, &ps)
			IF (pi.hPanelDC) THEN
			' Draw our fully assembled LCD display panel.
				bdr = pi.bwp 				' panels border width offset
				BitBlt (hDC, 0, 0, pi.nRight+2*bdr, pi.nBottom+2*bdr, pi.hPanelDC, 0, 0, $$SRCCOPY)
			END IF
			EndPaint (hWnd, &ps)
			RETURN

		CASE $$WM_DESTROY
			IF pi.hPanelDC THEN DeleteDC (pi.hPanelDC)
			hHeap = GetProcessHeap ()
			IF ppi THEN
				HeapFree (hHeap, 0, ppi)					' free the heap created in WM_CREATE
			END IF
			RETURN

		CASE $$LCD_GETPANELHEIGHT
			RETURN pi.nBottom + (pi.bwp*2) + (GetBorderWidth(hWnd)*2)

		CASE $$LCD_GETPANELWIDTH
			RETURN pi.nRight + (pi.bwp*2) + (GetBorderWidth(hWnd)*2)

		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT
END FUNCTION



FUNCTION Create5x7CellDC (hWnd, char)

' Purpose: -Build single 5x7 cell with specified character string.
' Entry  : -specified character string, single character only
' Returns: -a memory DC, up to caller to destroy

	PANELINFO pi
	RECT rc

	ppi = GetWindowLongA (hWnd, $$GWL_PDIDATA)
	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	' get character bitmap location
'	cp = ASC(char)
	IF char < 0x20 || char > 0x7e THEN char = 0x20
	cp = char - 0x20

	' get specifications
	PanelX 	= pi.panelX 					' width of panel
	dotX 		= pi.dotX 						' width of dot
	dotY 		= pi.dotY 						' height of dot
	PanelY 	= pi.panelY 					' nBottom, height of panel

	hDC = GetDC (hWnd)
	hCellDC = CreateCompatibleDC (hDC)
	hBrush1 = CreateSolidBrush (pi.colorset[0]) '<--Panel Background
	hBrush2 = CreateSolidBrush (pi.colorset[1]) '<--Dot off Color
	hBrush3 = CreateSolidBrush (pi.colorset[2]) '<--Dot on Color

	'create the bitmap
	hBitmap = CreateCompatibleBitmap (hDC, PanelX, PanelY)
	SelectObject (hCellDC, hBrush1)
	SelectObject (hCellDC, hBitmap)
	PatBlt (hCellDC, 0, 0, PanelX, PanelY, $$PATCOPY)
	SelectObject (hCellDC, hBitmap)

	FOR j = 0 TO 6 			' 7 ROWS
		FOR i = 0 TO 4 		' 5 COLUMNS
			SetRect (&rc, pi.gap+(i*dotX), pi.gap+(j*dotY), dotX+(i*dotX), dotY+(j*dotY))
			' dtrue meaning => dot on = 1, off =0 (flags are set in the character map)
			dtrue = EXTU(pi.cs.cm[cp].charset[j], 1, 7-i)	' get bit from character map
			IFZ dtrue THEN 																' test bit for status of dot, on/off?
				FillRect (hCellDC, &rc, hBrush2)
			ELSE
				FillRect (hCellDC, &rc, hBrush3)
			END IF
		NEXT i
	NEXT j

	' clean up...
	IF (hBitmap) THEN DeleteObject (hBitmap)
	IF (hBrush1) THEN DeleteObject (hBrush1)
	IF (hBrush2) THEN DeleteObject (hBrush2)
	IF (hBrush3) THEN DeleteObject (hBrush3)

	ReleaseDC (hWnd, hDC)

	RETURN hCellDC

END FUNCTION


FUNCTION CalculateBestfit (hPanel)

' calculate best fit for cells within window extents, seeds height for both
' cellDC and main panelDC

	PANELINFO pi
	RECT rc

	ppi = GetWindowLongA (hPanel, $$GWL_PDIDATA)
	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	' we need to calc our best fit for all the cells within window extents
	' window extents determines size of panelX width, once we have that,
	' panelX width, w/aspectRatio enforced determines panelY and window(hDC) height
'	GetClientRect (hPanel, &rc)
'	pw = rc.right \ (pi.nCols + 1)
	pw = pi.initWidth \ (pi.nCols + 1)

	' Size of the Panel determines dot dimensions...
	PanelX  = pw + pi.rightMargin 						' nRight, width of panel
	pi.dotX = PanelX \ 5 											' divide by 5 dots across
	pi.dotY = pi.dotX 												' dot size 1:1 ratio
	PanelY  = pi.dotY * 7 + pi.bottomMargin 	' nBottom, height of panel

	' save the adjusted height & width for the 5x7 cell
	pi.panelX = PanelX 												' return the overall width dimension of memDC
	pi.panelY = PanelY 												' return the overall height dimension of memDC

	' save the adjusted height and width for the Panel holder
	pi.nRight  = PanelX * pi.nCols 						'<-this should be very close to window extent
	pi.nBottom = PanelY * pi.nRows 						'<-this gets adjusted to fit rows

	IF ppi THEN	XstCopyMemory (&pi, ppi, SIZE(pi))

END FUNCTION


FUNCTION GetColorSet (hPanel)

	PANELINFO pi
	STATIC CLRDATA clrdata[]
	STATIC init

	ppi = GetWindowLongA (hPanel, $$GWL_PDIDATA)
	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	sc$ = TRIM$(UCASE$(pi.clrset)) ' get callers color set choice

	IFZ init THEN GOSUB InitColorData

	upp = UBOUND(clrdata[])

	FOR i = 0 TO upp
		st$ = clrdata[i].clr
		IF st$ = sc$ THEN
			pi.colorset[0] = RGB (clrdata[i].data[0], clrdata[i].data[1], clrdata[i].data[2])
			pi.colorset[1] = RGB (clrdata[i].data[3], clrdata[i].data[4], clrdata[i].data[5])
			pi.colorset[2] = RGB (clrdata[i].data[6], clrdata[i].data[7], clrdata[i].data[8])
			pi.bkcolor = pi.colorset[0]
			IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
			RETURN (1) 	' return success, a matching color string was found
		END IF
	NEXT i

	' use a default color set...offer orange
	pi.colorset[0] = RGB (clrdata[upp].data[0], clrdata[upp].data[1], clrdata[upp].data[2])
	pi.colorset[1] = RGB (clrdata[upp].data[3], clrdata[upp].data[4], clrdata[upp].data[5])
	pi.colorset[2] = RGB (clrdata[upp].data[6], clrdata[upp].data[7], clrdata[upp].data[8])
	pi.bkcolor = pi.colorset[0]
	IF ppi THEN XstCopyMemory (&pi, ppi, SIZE(pi))
	RETURN (-1)  ' return error, no such color in look up table

SUB InitColorData
	init = $$TRUE
' COLOR R,G,B R,G,B R,G,B
' NAME BK_GND DOT_OFF DOT_ON
	input$ = "BLUE,0,0,0,23,64,103,63,181,255, _
BLUE2,0,0,0,0,52,52,0,255,255, _
SALMON,0,0,0,103,64,23,255,181,63, _
GREEN,0,0,0,0,103,30,0,255,50, _
GREEN2,0,0,0,0,52,0,0,255,0, _
GREEN3,0,0,0,0,60,0,0,255,0, _
RED,0,0,0,103,30,0,255,60,0, _
RED2,0,0,0,52,0,0,255,0,0, _
RED3,0,0,0,60,0,0,255,0,0, _
LCD,164,176,70,148,157,64,0,0,0, _
LCD2,165,178,66,148,157,64,0,0,0, _
LCD3,148,156,66,165,181,66,0,0,0, _
WHITE,0,0,0,30,30,30,255,255,255, _
YELLOW,0,0,0,41,40,0,254,254,0, _
BLACK,95,168,241,85,138,198,0,0,0, _
BLACK2,90,170,247,82,138,198,0,0,0, _
BLACK3,93,165,231,55,136,230,0,0,0, _
ORANGE,206,150,49,189,134,33,0,0,0"

	DIM clrdata[0]
	template$ = "TYPE,STRING*32,XLONG,XLONG,XLONG,XLONG,XLONG,XLONG,XLONG,XLONG,XLONG"
	XstLoadCompositeDataArray (@clrdata[], input$, template$, @errornum)
END SUB
END FUNCTION

FUNCTION BuildPanel (hPanel, mode)

' Build Panel, fit rows & columns into window extents,
' PanelX and nbottom gets adjusted when enforcing aspect ratio

	RECT rc
	PANELINFO pi

	ppi = GetWindowLongA (hPanel, $$GWL_PDIDATA)

	' initialize color set using callers attributes
	lrt = GetColorSet (hPanel)

	' we need this first to dimension our main panel DC height and adjust our cell
	' height, panelY seeds @pdi.nBottom hDC extent (also later we fit window to hDC)
	IF mode <> 0 THEN CalculateBestfit (hPanel)

	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	' create our main display panel memory hDC...
	hDC = GetDC (hPanel)
	hPanelDC = CreateCompatibleDC (hDC)

	' paint panels(cell area) background
	hBrush = CreateSolidBrush (pi.bkcolor)

	' create the bitmap
	w = pi.nRight + 2 * pi.bwp
	h = pi.nBottom + 2 * pi.bwp
	hBitmap = CreateCompatibleBitmap (hDC, w, h)
	SelectObject (hPanelDC, hBrush)
	SelectObject (hPanelDC, hBitmap)
	PatBlt (hPanelDC, 0, 0, w, h, $$PATCOPY)

	x = pi.bwp
	y = pi.bwp
	FOR j = 1 TO pi.nRows 		' ROWS
		FOR i = 1 TO pi.nCols 	' COLUMNS
			ch = pi.txt.msg[j-1]{i-1}										' get a character from the message string
			h5X7DC = Create5x7CellDC (hPanel, ch) 			' get a new character Cell DC...
			BitBlt (hPanelDC, x, y, x+pi.panelX, y+pi.panelY, h5X7DC, 0, 0, $$SRCCOPY) ' copy cell to panel
			x = x + pi.panelX
			DeleteDC (h5X7DC) 													' destroy the temporary DC
		NEXT i
		y = y + pi.panelY
		x = pi.bwp
	NEXT j

	' clean up...
	IF (hBitmap) THEN DeleteObject (hBitmap)
	IF (hBrush) THEN DeleteObject (hBrush)

	ReleaseDC (hPanel, hDC)

	'return flag, and hold hPanelDC in private data memory
	IF (hPanelDC) THEN
		pi.hPanelDC = hPanelDC
		IF ppi THEN	XstCopyMemory (&pi, ppi, SIZE(pi))
		RETURN (1)
	ELSE
		RETURN (0)
	END IF

END FUNCTION


FUNCTION RegisterPanelEx ()

' Register the custom control and create panel holder window
' Class name: "displaypanel"
' Callback procedure: WndPanelProc()

	WNDCLASSEX wce

	hInst = GetModuleHandleA (NULL)

	'Register the control first time only please...
	IFZ GetClassInfoExA (hInst, &"displaypanel", &wce) THEN
		szAppName$ 				= "displaypanel"
		wce.cbSize 				= SIZE(wce)
		wce.style 				= $$CS_HREDRAW OR $$CS_VREDRAW
		wce.lpfnWndProc 	= &WndPanelProc()
		wce.cbClsExtra 		= 0
		wce.cbWndExtra 		= 4 		'<-private data pointer storage
		wce.hInstance 		= hInst
		wce.hIcon 				= NULL
		wce.hCursor 			= LoadCursorA (NULL, $$IDC_ARROW)
		wce.hbrBackground = GetStockObject (NULL) '%LTGRAY_BRUSH )
		wce.lpszMenuName 	= NULL
		wce.lpszClassName = &szAppName$
		wce.hIconSm 			= NULL

		'register this window class for the first time
		lResult = RegisterClassExA (&wce)
		IFZ lResult THEN
			RETURN (0)
		ELSE
			RETURN (lResult) 	' return atom
		END IF
	ELSE
		RETURN (-1) 			' already registered...
	END IF

END FUNCTION

FUNCTION ConfigureDisplayPanel (hPanel, row, col, colorset$, rmargin, bmargin, bwidth, gap)

	PANELINFO pi

	ppi = GetWindowLongA (hPanel, $$GWL_PDIDATA)
	IF ppi THEN	XstCopyMemory (ppi, &pi, SIZE(pi))

	pi.nRows        = row     			' save rows
	pi.nCols        = col     			' save columns
	pi.clrset       = colorset$     ' save color set ID string
	pi.rightMargin  = rmargin      	' save Right margin
	pi.bottomMargin = bmargin      	' save Bottom margin
	pi.bwp          = bwidth      	' save border witdh of panel
	pi.gap          = gap     			' save gap between dots

	IF ppi THEN	XstCopyMemory (&pi, ppi, SIZE(pi))

	' we can build our LCD display panel...
	SendMessageA (hPanel, $$LCD_REBUILD, 1, 0)

END FUNCTION

FUNCTION CreateDisplayPanel (hParent, msg$, ctrlID, x, y, cx, cy, bShow)

    ' Register the class for first time only...
    RegisterPanelEx ()

    hInst = GetModuleHandleA (NULL)

    exstyle = $$WS_EX_CLIENTEDGE OR $$WS_EX_DLGMODALFRAME
   'exstyle = $$WS_EX_CLIENTEDGE OR $$WS_EX_STATICEDGE

    hWnd = CreateWindowExA (exstyle, &"displaypanel", &msg$, $$WS_CHILD, x, y, cx, cy, hParent, ctrlID, hInst, NULL)

    ' show our window
    IF bShow THEN
        ' if initially hidden, then up to caller to show window...
        ShowWindow (hWnd, $$SW_SHOW)
    END IF

    RETURN hWnd

END FUNCTION

END PROGRAM

