'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' A LED control.
'
VERSION	"0.0001"
PROGRAM "led"
'
	IMPORT	"xst"   		' standard library	: required by most programs
	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library

' Bitmap representing LED bars to be 'lit'
' - 5
' | | 1 2
' - 6
' | | 3 4
' - 7
' Numeric definitions (As bits expressed in binary)"0b76543210"
$$LEDzero  = 0b10111110
$$LEDone   = 0b00010100
$$LEDtwo   = 0b11101100
$$LEDthree = 0b11110100
$$LEDfour  = 0b01010110
$$LEDfive  = 0b11110010
$$LEDsix   = 0b11011010
$$LEDseven = 0b00110100
$$LEDeight = 0b11111110
$$LEDnine  = 0b01110110
$$LEDhyphen= 0b01000000
$$LEDstroke= 0b10000000
$$LEDblank = 0

TYPE LEDINFO
	XLONG .nWidth 				' width of individual number in pixels
	XLONG .nHeight 				' height as above
	XLONG .ctlWidth				' width of entire led control (including border)
	XLONG .ctlHeight			' height of control (including border)
	XLONG .ledColor				' led color (rgb value)
	XLONG .bgColor				' background color (rgb value)
	XLONG .noborder				' $$TRUE = border not displayed, $$FALSE = border displayed (default)
	XLONG .borderStyle    ' border style
	XLONG .barThickness  	' thickness of individual LED bar
	STRING * 64 .value    ' number to display in led
	XLONG .hStatic				' handle to static control
	XLONG .tabstop
	XLONG .ghMemDCnum
	XLONG .ghBitmapNum
	XLONG .ghOldBitmapNum
	XLONG .ghMemDCDisp
	XLONG .ghBitmapDisp
	XLONG .ghOldBitmapDisp
	XLONG .cxBorder
	XLONG .cyBorder
END TYPE

EXPORT
DECLARE FUNCTION Led ()
END EXPORT

DECLARE FUNCTION PaintLED 				(LEDINFO ledinfo, xWidth, yHeight)
DECLARE FUNCTION MakeLEDBitMap 		(hWnd, LEDINFO li, xWidth, yHeight)
DECLARE FUNCTION DrawLEDbars 			(LEDINFO ledinfo, xFigure, xBitPattern)
DECLARE FUNCTION LedProc 					(hWnd, msg, wParam, lParam)

EXPORT
$$LEDCLASSNAME = "ledctrl"

' custom LED control messages
$$LED_SET_COLOR    		= 1025 		' set LED color (RGB value) in lParam (default is red)
$$LED_SET_BKCOLOR  		= 1026		' set back color (RGB value) in lParam (default is black)
$$LED_SET_BORDERSTYLE = 1027		' set border style in lParam (default is $$LED_BORDERSTYLE_SUNKEN) 

' border styles
$$LED_BORDERSTYLE_SUNKEN = 0xA
$$LED_BORDERSTYLE_BUMP   = 0x9
$$LED_BORDERSTYLE_ETCHED = 0x6
$$LED_BORDERSTYLE_RAISED = 0x5
$$LED_BORDERSTYLE_NONE   = 0
END EXPORT


FUNCTION Led ()
	WNDCLASS wc
	STATIC init
	
' do this once
	IF init THEN RETURN
	init = $$TRUE

	' Register control window class.
	className$       = "ledctrl"
	wc.style         = $$CS_HREDRAW OR $$CS_VREDRAW OR $$CS_GLOBALCLASS
  wc.lpfnWndProc   = &LedProc()
  wc.cbClsExtra    = 0
  wc.cbWndExtra    = 0    			' use 4 pre allocated bytes at offset $$GWL_USERDATA
  wc.hInstance     = GetModuleHandleA (0)
  wc.hIcon         = NULL
  wc.hCursor       = LoadCursorA (0, $$IDC_ARROW)
  wc.hbrBackground = $$COLOR_WINDOW + 1
  wc.lpszMenuName  = NULL
  wc.lpszClassName = &className$

  RETURN RegisterClassA (&wc)

END FUNCTION
'
' #####################
' #####  LedProc  #####
' #####################
'
'
'
FUNCTION LedProc (hWnd, msg, wParam, lParam)

	LEDINFO li
	
	PAINTSTRUCT ps
	CREATESTRUCT cs
	RECT rect
	SIZEAPI LEDsize, tBorder, stSize
	DRAWITEMSTRUCT dis

  pli = GetWindowLongA (hWnd, $$GWL_USERDATA)
  IFZ pli THEN
		pli = HeapAlloc (GetProcessHeap(), $$HEAP_ZERO_MEMORY, SIZE(li))
		IF pli THEN
' set pointer to link data in user data
			SetWindowLongA (hWnd, $$GWL_USERDATA, pli)
		ELSE
			RETURN
		END IF
  END IF
	RtlMoveMemory (&li, pli, SIZE(li))
	
	SELECT CASE msg
		
		CASE $$WM_CREATE :
			RtlMoveMemory (&cs, lParam, SIZE(cs))
			
			' to get the border sizes for the DrawEdge call
			li.cxBorder = GetSystemMetrics($$SM_CXEDGE)
			li.cyBorder = GetSystemMetrics($$SM_CYEDGE)
	
			' size of individual numbers
			li.nHeight = cs.cy
			li.nWidth = cs.cx
			
			li.ctlHeight = li.cyBorder*2 + li.nHeight 		' 1 number high
			
			v$ = CSTRING$(cs.lpszName)
			li.value = v$
			len = LEN(v$)
			li.ctlWidth = li.cxBorder*2 + li.nWidth*len 	' len numbers wide
			
			' create static control to display LED information.
			li.hStatic = CreateWindowExA (NULL, &"STATIC", NULL, $$WS_CHILDWINDOW OR $$WS_VISIBLE OR $$SS_OWNERDRAW, cs.x, cs.y, li.ctlWidth, li.ctlHeight, hWnd, 0, cs.hInstance, NULL)

			' resize parent window
			MoveWindow (hWnd, cs.x, cs.y, li.ctlWidth, li.ctlHeight, 1)	
			
			' set default colors
			li.ledColor = RGB (255, 0, 0)
			li.bgColor = RGB (0, 0, 0)
			
			' set default border style
			li.borderStyle = $$LED_BORDERSTYLE_SUNKEN
			
			' create LED number bitmap
			MakeLEDBitMap (hWnd, @li, li.nWidth, li.nHeight)
			
			' create bitmap display to be held by the ownerdraw static control
			hDC = GetDC (hWnd)
			li.ghMemDCDisp = CreateCompatibleDC (NULL)
			li.ghBitmapDisp = CreateCompatibleBitmap (hDC, li.nWidth*len, li.nHeight)
			li.ghOldBitmapDisp = SelectObject (li.ghMemDCDisp, li.ghBitmapDisp)
			ReleaseDC (hWnd, hDC)
			
			' BitBlt numbers to memory bitmap
			PaintLED (@li, li.nWidth, li.nHeight) 
			
			RtlMoveMemory (pli, &li, SIZE(li))
			
		CASE $$WM_SETTEXT :
      text$ = NULL$(256)
			RtlMoveMemory (&text$, lParam, SIZE(text$))
			li.value = CSIZE$ (text$)
			PaintLED (@li, li.nWidth, li.nHeight) 
			InvalidateRect (hWnd, NULL, 0) 
			RtlMoveMemory (pli, &li, SIZE(li))
			
		CASE $$WM_PAINT :
			hDC = BeginPaint (hWnd, &ps)
			GetClientRect (hWnd, &rect)
			IF li.borderStyle THEN 
				DrawEdge (hDC, &rect, li.borderStyle, $$BF_RECT)
			ELSE
				hBrush = CreateSolidBrush (li.bgColor)
				hOldBrush = SelectObject (hDC, hBrush)
				FillRect (hDC, &rect, hBrush)
				SelectObject (hDC, hOldBrush)
				DeleteObject (hBrush)
			END IF
			len = LEN(li.value)
			BitBlt (hDC, rect.left + li.cxBorder, rect.top + li.cyBorder, li.nWidth*len, li.nHeight, li.ghMemDCDisp, 0, 0, $$SRCCOPY)
			EndPaint (hWnd, &ps)
			
		CASE $$WM_DESTROY :
			IF li.ghMemDCnum THEN
				DeleteObject (SelectObject (li.ghMemDCnum, li.ghOldBitmapNum))
				DeleteObject (li.ghMemDCnum)
				li.ghMemDCnum = 0
			END IF
			IF li.ghMemDCDisp THEN
				DeleteObject (SelectObject (li.ghMemDCDisp, li.ghOldBitmapDisp))
				DeleteObject (li.ghMemDCDisp)
				li.ghMemDCDisp = 0
			END IF
			
		CASE $$LED_SET_BORDERSTYLE :
			li.borderStyle = lParam
			RtlMoveMemory (pli, &li, SIZE(li))	
			InvalidateRect (hWnd, NULL, 0) 
			
		CASE $$LED_SET_COLOR :
			li.ledColor = lParam
			MakeLEDBitMap (hWnd, @li, li.nWidth, li.nHeight)
			PaintLED (@li, li.nWidth, li.nHeight) 
			RtlMoveMemory (pli, &li, SIZE(li))
			InvalidateRect (hWnd, NULL, 0) 
			
		CASE $$LED_SET_BKCOLOR :
			li.bgColor = lParam
			MakeLEDBitMap (hWnd, @li, li.nWidth, li.nHeight)
			PaintLED (@li, li.nWidth, li.nHeight) 
			RtlMoveMemory (pli, &li, SIZE(li))
			InvalidateRect (hWnd, NULL, 0) 
			
			
		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
' #########################
' #####  PaintLED ()  #####
' #########################
'
FUNCTION PaintLED (LEDINFO ledinfo, xWidth, yHeight)
	STRING tString

	tString = ledinfo.value
	len = LEN(tString)

	FOR i = 1 TO len
		a$ = MID$(tString, i, 1)
		SELECT CASE a$
			CASE "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" : pos = XLONG (a$)
			CASE "-" : pos = 10
			CASE "_" : pos = 11
			CASE ELSE : pos = 12
		END SELECT
		BitBlt (ledinfo.ghMemDCDisp, (i-1)*xWidth, 0, xWidth, yHeight, ledinfo.ghMemDCnum, pos*xWidth, 0, $$SRCCOPY)
	NEXT i

END FUNCTION

FUNCTION MakeLEDBitMap (hWnd, LEDINFO ledinfo, xWidth, yHeight)

	IF ledinfo.ghMemDCnum THEN
		DeleteObject (SelectObject (ledinfo.ghMemDCnum, ledinfo.ghOldBitmapNum))
		DeleteObject (ledinfo.ghMemDCnum)
		ledinfo.ghMemDCnum = 0
	END IF

	hDC = GetDC (hWnd)
	ledinfo.ghMemDCnum = CreateCompatibleDC(NULL)
	ledinfo.ghBitmapNum = CreateCompatibleBitmap (hDC, xWidth*13, yHeight)
	ledinfo.ghOldBitmapNum = SelectObject (ledinfo.ghMemDCnum, ledinfo.ghBitmapNum)
	
	' create pen & brush for background
	hPen = CreatePen ($$PS_SOLID, 1, ledinfo.bgColor)
	hBrush = CreateSolidBrush (ledinfo.bgColor)
	hOldPen = SelectObject (ledinfo.ghMemDCnum, hPen)
	hOldBrush = SelectObject (ledinfo.ghMemDCnum, hBrush)
	Rectangle (ledinfo.ghMemDCnum, 0, 0, xWidth*13+1, yHeight)
	
	' create pen and brush for LED
	hPen = CreatePen ($$PS_SOLID, 1, ledinfo.ledColor)
	hBrush = CreateSolidBrush (ledinfo.ledColor)
	SelectObject (ledinfo.ghMemDCnum, hPen)
	SelectObject (ledinfo.ghMemDCnum, hBrush)
	ledinfo.barThickness = xWidth\10 '12

	DrawLEDbars (ledinfo, 0, $$LEDzero)
	DrawLEDbars (ledinfo, 1, $$LEDone)
	DrawLEDbars (ledinfo, 2, $$LEDtwo)
	DrawLEDbars (ledinfo, 3, $$LEDthree)
	DrawLEDbars (ledinfo, 4, $$LEDfour)
	DrawLEDbars (ledinfo, 5, $$LEDfive)
	DrawLEDbars (ledinfo, 6, $$LEDsix)
	DrawLEDbars (ledinfo, 7, $$LEDseven)
	DrawLEDbars (ledinfo, 8, $$LEDeight)
	DrawLEDbars (ledinfo, 9, $$LEDnine)
	DrawLEDbars (ledinfo, 10, $$LEDhyphen)
	DrawLEDbars (ledinfo, 11, $$LEDstroke)
	
	' clean Up
	IF hPen THEN DeleteObject (SelectObject (ledinfo.ghMemDCnum, hOldPen))
	IF hBrush THEN DeleteObject (SelectObject (ledinfo.ghMemDCnum, hOldBrush))
	
	ReleaseDC (hWnd, hDC)

END FUNCTION

FUNCTION DrawLEDbars (LEDINFO ledinfo, xFigure, xBitPattern)

	POINT pt[]

	DIM pt[7]
	
	offset = xFigure * ledinfo.nWidth
'	padding = MAX (ledinfo.barThickness\4, 1)
	padding = ledinfo.barThickness
	centerBarThickness = 1.5 * ledinfo.barThickness
	
	' balance center bar by making it an odd number.
	IF INT(centerBarThickness\2)*2 = centerBarThickness THEN INC centerBarThickness
	cbUpDown = (centerBarThickness -1)\2
	halfHeight = ledinfo.nHeight\2
	
	' upper left bar
	IF xBitPattern{1, 1} THEN
		pt[0].x = offset + padding + 1 : pt[0].y = padding + 2
		pt[1].x = pt[0].x : pt[1].y = halfHeight - 1
		pt[2].x = pt[1].x + ledinfo.barThickness : pt[2].y = pt[1].y - ledinfo.barThickness
		pt[3].x = pt[2].x : pt[3].y = pt[0].y + ledinfo.barThickness
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF
	' upper right bar
	IF xBitPattern{1, 2} THEN
		pt[0].x = offset + ledinfo.nWidth - (padding + 1) : pt[0].y = padding + 2
		pt[1].x = pt[0].x : pt[1].y = halfHeight - 1
		pt[2].x = pt[1].x - ledinfo.barThickness : pt[2].y = pt[1].y - ledinfo.barThickness
		pt[3].x = pt[2].x : pt[3].y = pt[0].y + ledinfo.barThickness
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF
	' lower left bar
	IF xBitPattern{1, 3} THEN
		pt[0].x = offset + padding + 1 : pt[0].y = halfHeight + 1
		pt[1].x = pt[0].x + ledinfo.barThickness : pt[1].y = pt[0].y + ledinfo.barThickness
		pt[2].x = pt[1].x : pt[2].y = ledinfo.nHeight - (padding + 2 + ledinfo.barThickness)
		pt[3].x = pt[0].x : pt[3].y = pt[2].y + ledinfo.barThickness
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF
	' lower right bar
	IF xBitPattern{1, 4} THEN
		pt[0].x = offset + ledinfo.nWidth - (padding + 1) : pt[0].y = halfHeight + 1
		pt[1].x = pt[0].x : pt[1].y = ledinfo.nHeight - (padding + 2)
		pt[2].x = pt[1].x - ledinfo.barThickness : pt[2].y = ledinfo.nHeight - (padding + 2 + ledinfo.barThickness)
		pt[3].x = pt[2].x : pt[3].y = pt[0].y + ledinfo.barThickness
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF
	' top bar
	IF xBitPattern{1, 5} THEN
		pt[0].x = offset + padding + 2 : pt[0].y = padding + 1
		pt[1].x = offset + ledinfo.nWidth - (padding + 2) : pt[1].y = pt[0].y
		pt[2].x = pt[1].x - ledinfo.barThickness : pt[2].y = pt[0].y + ledinfo.barThickness
		pt[3].x = pt[0].x + ledinfo.barThickness : pt[3].y = pt[2].y
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF
	' center bar
	IF xBitPattern{1, 6} THEN
		pt[0].x = offset + padding + 2 : pt[0].y = halfHeight
		pt[3].x = offset + ledinfo.nWidth - (padding + 2) : pt[3].y = pt[0].y
		pt[1].x = pt[0].x + cbUpDown : pt[1].y = pt[0].y - cbUpDown
		pt[2].x = pt[3].x - cbUpDown : pt[2].y = pt[3].y - cbUpDown
		pt[4].x = pt[3].x - cbUpDown : pt[4].y = pt[3].y + cbUpDown
		pt[5].x = pt[0].x + cbUpDown : pt[5].y = pt[0].y + cbUpDown
		Polygon (ledinfo.ghMemDCnum, &pt[0], 6)
	END IF
	' bottom bar
		IF xBitPattern{1, 7} THEN
		pt[0].x = offset + padding + 2 : pt[0].y = ledinfo.nHeight - (padding + 1)
		pt[1].x = offset + ledinfo.nWidth - (padding + 2) : pt[1].y = pt[0].y
		pt[2].x = pt[1].x - ledinfo.barThickness : pt[2].y = pt[0].y - ledinfo.barThickness
		pt[3].x = pt[0].x + ledinfo.barThickness : pt[3].y = pt[2].y
		Polygon (ledinfo.ghMemDCnum, &pt[0], 4)
	END IF

END FUNCTION

END PROGRAM
