PROGRAM "animatewindow"
VERSION "1.02"
'
' animatewindow - demo of API AnimateWindow
' Copyright  GPL 2008-2009 Guy "gl" Lonne.
'
' ***** Description *****
' demo of the API AnimateWindow (hwnd, millisec, flags)
' with several constants:
' - $$AW_HOR_POSITIVE: animate the window from left to right
' - $$AW_HOR_NEGATIVE: animate the window from right to left
' - $$AW_VER_POSITIVE: animate the window from top to bottom
' - $$AW_VER_NEGATIVE: animate the window from bottom to top
' - $$AW_CENTER      : make the window appear
'                      - to "explode" on show (with $$AW_ACTIVATE),
'                      - to collapse inward on close (with $$AW_HIDE).
'
' ***** Versions *****
' 1.00-gl-21may08: creation.
' 1.01-gl-05jan09: GUI prototype with VIXEN v1.99;
' generation switches that are on:
' - Use Windows XP Style
' - Use Debugging functions
' - Use Windows API
' - Meticulous Cleanup
'
' 1.02-gl-07jan09: corrected Noel Pil's bug.
' - Bug: Each animation test adds a new app icon in Windows' taskbar,
'   cleaned up on closing.
' - Correction: The animated window class must be unregistered every time
'   the animated window is destroyed.
'
'
' ##############################
' #####  Import Libraries  #####
' ##############################
'
' ******  Comment libraries in/out as needed  *****
'
' Win32API DLL headers
'
	IMPORT "kernel32"   ' operating system
'
' ---Note: import gdi32 BEFORE shell32 and user32
	IMPORT "gdi32"      ' Graphic Device Interface
	IMPORT "shell32"    ' interface to the operating system
	IMPORT "user32"     ' Windows management
'	IMPORT "advapi32"   ' advanced API: security, services, registry ...
'
' ---Note: import comctl32 BEFORE comdlg32
	IMPORT "comctl32"   ' common controls; ==> initialize w/ InitCommonControls ()
'
' xblite DLL headers
' ---Note: dll (IMPORT "dll") and static libraries (IMPORT "dll_s")
'          can't be mixed here as it can cause a program crash.
'
	IMPORT "xst_s"      ' xblite Standard Library (static link)
	IMPORT "xsx_s"      ' xblite Extended Standard Library (static link)
'	IMPORT "xio_s"      ' console (static link)
'
'
'
' ########################################
' #####  Composite Type Definitions  #####
' ########################################
'
'
'
'
' ############################################
' #####  Internal Function Declarations  #####
' ############################################
'
'------------------------------------------------------------------
'
DECLARE FUNCTION Entry () ' program entry point

DECLARE FUNCTION CleanUp () ' application cleanup

DECLARE FUNCTION CreateWindows () ' create the windows of the application


DECLARE FUNCTION GuiSetFont (hCtr, hFont) ' apply font to control
DECLARE FUNCTION GuiSetText (hCtr, text$) ' set the text for a control
DECLARE FUNCTION GuiTellApiError (msg$) ' displays an API error message
DECLARE FUNCTION GuiTellRunError (msg$) ' displays an execution error message

DECLARE FUNCTION InitGui () ' initialize win32 controls and libraries
DECLARE FUNCTION InitProgram () ' application setup

DECLARE FUNCTION KillFont (@hFont) ' release a font created by NewFont

DECLARE FUNCTION MessageLoop () ' main message loop

DECLARE FUNCTION NewChild (className$, text$, style, x, y, w, h, parent, id, exStyle) ' create a child window
DECLARE FUNCTION NewFont (fontName$, pointSize, weight, italic, underline, strikeOut) ' create a new font
DECLARE FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle) ' create a window

DECLARE FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$) ' register the window class

DECLARE FUNCTION WndProc (hwnd, msg, wParam, lParam) ' callback function for window #winMain

DECLARE FUNCTION winAnimated_CallBack (hwnd, msg, wParam, lParam) ' callback function for window #winAnimated
'
'
' ############################################
' #####  External Function Declarations  #####
' ############################################
'
' (None)

'
'
' #########################################
' #####  Shared Constant Definitions  #####
' #########################################
'
'
' ***** Constants used as control identificators for window #winMain *****
' control identificators for #winMain
$$grpHorizontal   = 9000	' button group 'Horizontal Effect'
$$grpVertical     = 9001	' button group 'Vertical Effect'

$$radHOR_POSITIVE = 9002	' radio button 'left to right'
$$radHOR_NEGATIVE = 9003	' radio button 'right to left'
$$radVER_POSITIVE = 9004	' radio button 'top to bottom'
$$radVER_NEGATIVE = 9005	' radio button 'bottom to top'
$$radAW_CENTER    = 9006	' radio button 'Center'

$$btnOK           = 9007	' button '&OK'
$$btnExit         = 9008	' button 'E&xit'
'
' ***** Constants used as control identificators for window #winAnimated *****
' control identificators for #winAnimated
$$winAnimated_btnClose = 9014	' button '&Close'
'
'
' ##########################################
' #####  Shared Variable Declarations  #####
' ##########################################
'

'
'
' #################################
' #####  Program Entry Point  #####
' #################################
'
FUNCTION Entry () ' Program Entry Point

	STATIC	entry

	IF entry THEN RETURN                  ' enter once
	entry =  $$TRUE                       ' enter occured

'	XioCreateConsole (title$, 50)         ' create console, if console is not wanted, comment out this line

	IF InitProgram ()   THEN XstAbend ("Entry: Can't initialize application")
	IF InitGui ()       THEN XstAbend ("Entry: Can't initialize Windows GUI")
	IF CreateWindows () THEN XstAbend ("Entry: Can't create the window(s)")

	MessageLoop ()                        ' the main message loop

	CleanUp ()                            ' application cleanup
'	XioFreeConsole ()                     ' free console
	QUIT (0)                              ' stop run

END FUNCTION

FUNCTION CleanUp () ' application cleanup
	SHARED hInst
	SHARED hFontDefault ' default control font

	KillFont (@hFontDefault) ' release font default control font

	IF #winAnimated THEN
		DestroyWindow (#winAnimated) ' destroy the window
	END IF
	#winAnimated = 0
	UnregisterClassA (&#winAnimated_class$, hInst) ' unregister the window class

	IF #winMain THEN DestroyWindow (#winMain)
	#winMain = 0
	UnregisterClassA (&#winMain_class$, hInst)

	RETURN $$FALSE ' Always OK!

END FUNCTION

FUNCTION CreateWindows () ' create the windows of the application
	SHARED hInst

	WNDCLASSEX wcEx		' window class

	' ***************** Begin window #winMain Generation *****************

	' register the window class for window #winMain
	#winMain_class$ = "winMainClass"
	addrWndProc = &WndProc ()
	icon$ = ""
	menu$ = ""
	SetLastError (0) ' clear the last-error code
	bErr = RegisterWinClass (#winMain_class$, addrWndProc, icon$, menu$) ' register the window class
	IF bErr THEN  ' error
		msg$ = "RegisterWinClass: Can't register the window class"
		GuiTellApiError (msg$)
	END IF
	' create window #winMain
	titleBar$ = "Demo API AnimateWindow, v" + VERSION$ (0)
	style     = $$WS_OVERLAPPEDWINDOW
	#winMain = NewWindow (#winMain_class$, titleBar$, style, 0, 0, 386, 346, 0) ' create window #winMain
	IFZ #winMain THEN ' error
		msg$ = "NewWindow: Can't create window #winMain"
		XstAlert (msg$)
		RETURN $$TRUE ' error
	END IF
	' ***************** End window #winMain Generation *****************

	XstCenterWindow (#winMain) ' auto-center the window #winMain
	ShowWindow (#winMain, $$SW_SHOWNORMAL) ' show the window #winMain

	RETURN $$FALSE ' OK!

END FUNCTION



FUNCTION GuiSetFont (hCtr, hFont) ' apply font to control

	IFZ hCtr THEN
		XstAlert ("GuiSetFont: null control handle")
		RETURN $$TRUE ' error
	END IF
	IFZ hFont THEN
		XstAlert ("GuiSetFont: null font handle")
		RETURN $$TRUE ' error
	END IF
	' $$WM_SETFONT does not return a value
	SendMessageA (hCtr, $$WM_SETFONT, hFont, 1) ' 1 = refresh
	RETURN $$FALSE ' OK!

END FUNCTION

FUNCTION GuiSetText (hCtr, text$) ' set the text for a control
	' hCtr = the handle to the control
	' text$ = the text to set
	'
	' Returns
	' - an error flag: $$TRUE = error, $$FALSE = OK!

	IFZ hCtr THEN RETURN $$TRUE ' error

	SetLastError (0) ' clear the last-error code
	ret = SetWindowTextA (hCtr, &text$)
	IFZ ret THEN RETURN $$TRUE ' error

	RETURN $$FALSE ' OK!

END FUNCTION

FUNCTION GuiTellApiError (msg$)		' display an API error message
	'
	' Returns
	' - an error flag: $$TRUE = error, $$FALSE = OK!
	'
	' Usage:
'	SetLastError (0)		' clear the last-error code
'	hDefaultFont = GetStockObject ($$ANSI_VAR_FONT)		' get the ANSI default font
'	IFZ hDefaultFont THEN		' error
'		msg$ = "Can't get the ANSI default font with 'GetStockObject ($$ANSI_VAR_FONT)'"
'		GuiTellApiError (msg$)
'	END IF

	' get the last error code, then clear it
	errNum = GetLastError ()
	SetLastError (0)
	IFZ errNum THEN RETURN $$FALSE ' was OK!

	fmtMsg$ = "Last error code:" + STR$ (errNum) + " "

	lenBuf = 1020
	szBuf$ = NULL$ (lenBuf) ' fill szBuf$ with (lenBuf + 1) null chars

	' set up FormatMessageA arguments
	dwFlags = $$FORMAT_MESSAGE_FROM_SYSTEM | $$FORMAT_MESSAGE_IGNORE_INSERTS

	' format a message string
	ret = FormatMessageA (dwFlags, 0, errNum, 0, &szBuf$, lenBuf, 0)
	IFZ ret THEN
		fmtMsg$ = fmtMsg$ + "(unknown)"
	ELSE
		fmtMsg$ = fmtMsg$ + CSTRING$ (&szBuf$)
	END IF
	fmtMsg$ = fmtMsg$ + "\r\n" + msg$
	IFZ msg$ THEN fmtMsg$ = fmtMsg$ + "Windows API error"

	XstGetOSName (@os$)
	XstGetOSVersion (@major, @minor, @platformId, @version$, @platform$)
	text$ = "\r\nOS: " + os$ + STR$ (major) + "." + STRING$ (minor) + " " + platform$
	fmtMsg$ = fmtMsg$ + text$

	' set up MessageBoxA arguments
	title$ = PROGRAM$ (0) + "-API Error"
	hWnd = GetActiveWindow ()
	MessageBoxA (hWnd, &fmtMsg$, &title$, $$MB_ICONSTOP)

	RETURN $$TRUE ' an error really occurred!

END FUNCTION

FUNCTION GuiTellRunError (msg$)		' display the run-time error message
	'
	' Returns
	' - an error flag: $$TRUE = error, $$FALSE = OK!
	'
	' Usage:
'	errNum = ERROR ($$FALSE) ' clear the last-error code
'	inFile = OPEN (#inFile$, $$RD)
'	IF inFile < 3 THEN
'		msg$ = "Can't open input file " + #inFile$
'		GuiTellRunError (msg$)
'	END IF

	' get current error, then clear it
	errNum = ERROR ($$FALSE)
	IFZ errNum THEN RETURN $$FALSE ' was OK!

	fmtMsg$ = "Error code:" + STR$ (errNum) + " " + ERROR$ (errNum) + "\r\n" + msg$
	IFZ msg$ THEN fmtMsg$ = fmtMsg$ + "Xblite library error"

	' set up MessageBoxA arguments
	title$ = PROGRAM$ (0) + "-Execution Error"
	hWnd = GetActiveWindow ()
	MessageBoxA (hWnd, &fmtMsg$, &title$, $$MB_ICONSTOP)

	RETURN $$TRUE ' an error really occurred!

END FUNCTION

FUNCTION InitGui () ' initialize win32 controls and libraries

	SHARED hInst
	SHARED hFontDefault

	INITCOMMONCONTROLSEX iccex

	hInst = GetModuleHandleA (0)  ' get current instance handle
	IFZ hInst THEN XstAbend ("InitGui: Can't get current instance handle")

	' initialize the specific common controls classes from the common
	' control dynamic-link library
	iccex.dwSize = SIZE(iccex)

	' $$ICC_ANIMATE_CLASS      : animate
	' $$ICC_BAR_CLASSES        : toolbar, statusbar, trackbar, tooltips
	' $$ICC_COOL_CLASSES       : rebar (coolbar) control
	' $$ICC_DATE_CLASSES       : month picker, date picker, time picker, updown
	' $$ICC_HOTKEY_CLASS       : hotkey
	' $$ICC_INTERNET_CLASSES   : WIN32_IE >= 0x0400
	' $$ICC_LISTVIEW_CLASSES   : listview, header
	' $$ICC_PAGESCROLLER_CLASS : page scroller (WIN32_IE >= 0x0400)
	' $$ICC_PROGRESS_CLASS     : progress
	' $$ICC_TAB_CLASSES        : tab, tooltips
	' $$ICC_TREEVIEW_CLASSES   : treeview, tooltips
	' $$ICC_UPDOWN_CLASS       : updown
	' $$ICC_USEREX_CLASSES     : comboex
	' $$ICC_WIN95_CLASSES      : everything else
	iccex.dwICC  = $$ICC_ANIMATE_CLASS    | _
		          $$ICC_BAR_CLASSES        | _
		          $$ICC_COOL_CLASSES       | _
		          $$ICC_DATE_CLASSES       | _
		          $$ICC_HOTKEY_CLASS       | _
		          $$ICC_INTERNET_CLASSES   | _
		          $$ICC_LISTVIEW_CLASSES   | _
		          $$ICC_NATIVEFNTCTL_CLASS | _
		          $$ICC_PAGESCROLLER_CLASS | _
		          $$ICC_PROGRESS_CLASS     | _
		          $$ICC_TAB_CLASSES        | _
		          $$ICC_TREEVIEW_CLASSES   | _
		          $$ICC_UPDOWN_CLASS       | _
		          $$ICC_USEREX_CLASSES     | _
		          $$ICC_WIN95_CLASSES

	InitCommonControlsEx (&iccex)

	hFontDefault = GetStockObject ($$ANSI_VAR_FONT) ' get the ANSI default font
	IFZ hFontDefault THEN ' error
		msg$ = "InitGui: Can't get the ANSI default font with 'GetStockObject ($$ANSI_VAR_FONT)'"
		XstAlert (msg$)
	END IF
END FUNCTION
'
'
' ################################
' #####  Internal Functions  #####
' ################################
'
FUNCTION InitProgram () ' application setup
	'
	' Returns
	' - an error flag: $$TRUE = error, $$FALSE = OK!

	Xst () ' initialize Xblite Standard Library
	Xsx () ' initialize Xblite Extended Standard Library
'	Xio () ' initialize console

	RETURN $$FALSE		'OK!

END FUNCTION

FUNCTION KillFont (@hFont) ' release a font created by NewFont

	IFZ hFont THEN RETURN $$FALSE ' OK!

	SetLastError (0) ' clear the last-error code
	ret = DeleteObject (hFont) ' release the font
	IFZ ret THEN ' error
		msg$ = "DeleteObject: Can't release the font"
		GuiTellApiError (msg$)
		RETURN $$TRUE ' error
	END IF
	hFont = 0
	RETURN $$FALSE ' OK!

END FUNCTION

FUNCTION MessageLoop ()		' main message loop
	'
	' Returns
	' - an error flag: $$TRUE = error, $$FALSE on quit

	MSG msg		' will be sent to window callback function when an event occurs

	' reserved to a stand-alone application
	IF LIBRARY (0) THEN RETURN		' main program executes message loop

	' supervise system messages until
	' - the user decides to leave the application (ret = 0)
	' - an error occurred (ret = -1)
	DO		' the message loop
		' retrieve next message from queue
		ret = GetMessageA (&msg, 0, 0, 0)
		SELECT CASE ret
			CASE  0 : RETURN $$FALSE	' received a quit message
			CASE -1 : RETURN $$TRUE		' error
			CASE ELSE
				' deal with window messages
				hwnd = GetActiveWindow ()
				'
				' Process accelerator keys for menu commands
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN
					' send only non-dialog messages
					' translate virtual-key messages into character messages
					' ex.: SHIFT + a is translated as "A"
					TranslateMessage (&msg)
					'
					' send message to window callback function
					DispatchMessageA (&msg)
				END IF
		END SELECT
	LOOP		' forever

END FUNCTION

FUNCTION NewChild (className$, text$, style, x, y, w, h, hParent, idc, exStyle) ' create a child window
	'
	' Arguments
	' - className$: control's class
	' - text$     : title
	' - style     : style
	' - x         : Left
	' - y         : Top
	' - w         : Width
	' - h         : Height
	' - hParent   : window or parent control
	' - idc       : control ID
	' - exStyle   : extended style
	'
	' Returns
	' - The child window's handle if OK!, 0 = error

	SHARED hInst

	IFZ TRIM$ (className$) THEN
		msg$ = "NewChild: Can't create the child window " + text$ + ", with an empty control class"
		XstAlert (msg$)
		RETURN 0		' error
	END IF
	IFZ hParent THEN
		msg$ = "NewChild: Can't create the child window " + text$ + ", control class " + className$ + " without a parent"
		XstAlert (msg$)
		RETURN 0		' error
	END IF
	' create the child window
	newStyle = style | $$WS_CHILD | $$WS_VISIBLE

	SetLastError (0) ' clear the last-error code
	hCtr = CreateWindowExA (exStyle, &className$, &text$, newStyle, x, y, w, h, hParent, idc, hInst, 0)
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create the child window " + text$ + " with CreateWindowExA, control class " + className$
		XstAlert (msg$)
	END IF
	RETURN hCtr

END FUNCTION

FUNCTION NewFont (fontName$, pointSize, weight, italic, underline, strikeOut) ' get a predefined GUI font
	' Returns the font handle if OK!, 0 = error

	LOGFONT font

	IFZ fontName$ THEN
		XstAlert ("NewFont: empty font face")
		RETURN 0 ' error
	END IF
	SetLastError (0) ' clear the last-error code
	hDefaultGuiFont = GetStockObject ($$DEFAULT_GUI_FONT) ' get a handle to the predefined GUI font
	IFZ hDefaultGuiFont THEN ' error
		msg$ = "GetStockObject: Can't get a handle to the predefined GUI font"
		GuiTellApiError (msg$)
		RETURN 0 ' error
	END IF
	' hDefaultGuiFont provides with a well-formed font structure
	SetLastError (0) ' clear the last-error code
	bytes = GetObjectA (hDefaultGuiFont, SIZE (font), &font) ' allocate structure font
	IFZ bytes THEN ' error
		msg$ = "GetObjectA: Can't allocate structure font"
		GuiTellApiError (msg$)
		RETURN 0 ' error
	END IF
	SetLastError (0) ' clear the last-error code
	bErr = KillFont (@hDefaultGuiFont) ' release the default GUI font
	IF bErr THEN  ' error
		msg$ = "KillFont: Can't release the default GUI font"
		GuiTellApiError (msg$)
	END IF
	' set struct font
	font.faceName = fontName$

	IF pointSize THEN ' character height is specified
		' Windows expects the font size to be in pixels and negative
		SetLastError (0) ' clear the last-error code
		hdc = GetDC ($$HWND_DESKTOP) ' get the desktop context's handle
		IFZ hdc THEN ' error
			msg$ = "GetDC: Can't get the desktop context's handle"
			GuiTellApiError (msg$)
			RETURN 0 ' error
		END IF
		'
		font.height = (pointSize * GetDeviceCaps (hdc, $$LOGPIXELSY)) / (-72)
		ReleaseDC ($$HWND_DESKTOP, hdc) ' release the handle of the desktop context
	END IF
	IF weight > 0 THEN
		font.weight = weight
	ELSE
		font.weight = 0
	END IF
	IF italic THEN
		font.italic = 1
	ELSE
		font.italic = 0
	END IF
	IF underline THEN
		font.underline = 1
	ELSE
		font.underline = 0
	END IF
	IF strikeOut THEN
		font.strikeOut = 1
	ELSE
		font.strikeOut = 0
	END IF
	SetLastError (0) ' clear the last-error code
	hFont = CreateFontIndirectA (&font) ' create the logical font
	IFZ hFont THEN ' error
		msg$ = "CreateFontIndirectA: Can't create the logical font"
		GuiTellApiError (msg$)
		RETURN 0 ' error
	END IF
	RETURN hFont

END FUNCTION

FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle) ' create a window
	'
	' Arguments
	' - className$: window's class
	' - titleBar$ : title
	' - style     : style
	' - x         : Left
	' - y         : Top
	' - w         : Width
	' - h         : Height
	' - exStyle   : extended style
	'
	' Returns
	' - The window's handle if OK!, 0 = error

	SHARED hInst

	SetLastError (0) ' clear the last-error code
	hWin = CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)
	IFZ hWin THEN
		msg$ = "NewWindow: Can't create the window " + titleBar$ + " with CreateWindowExA, window class " + className$
		XstAlert (msg$)
	END IF
	RETURN hWin

END FUNCTION

FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$) ' register the window class
	' An application icon should come in two versions:
	' - 16x16 pixels for the window itself. It is also used except
	'   for the Large View ('wc.hIconSmor wc.hIconSm)
	'
	' - 32x32 pixels for the Large View of certain windows such as
	'   My Documents or Windows Explorer(wc.hIcon or wc.hIcon)

	SHARED hInst

	WNDCLASSEX wcEx		' window class

	wcEx.cbSize          = SIZE (wcEx)
	wcEx.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wcEx.lpfnWndProc     = addrWndProc
	wcEx.cbClsExtra      = 0                 ' no extra bytes after the window class
	wcEx.cbWndExtra      = 0                 ' structure or the window instance
	wcEx.hInstance       = hInst

	IF icon$ THEN
		wcEx.hIcon    = LoadIconA (hInst, &icon$)
		'wcEx.hIcon   = LoadImageA (hInst, &icon$, $$IMAGE_ICON, 32, 32, $$LR_DEFAULTSIZE)
		'
		wcEx.hIconSm  = LoadIconA (hInst, &icon$)
		'wcEx.hIconSm = LoadImageA (hInst, &icon$, $$IMAGE_ICON, 16, 16, $$LR_DEFAULTSIZE)
	END IF
	wcEx.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wcEx.hbrBackground   = $$COLOR_BTNFACE + 1
	IF menu$ THEN wcEx.lpszMenuName = &menu$
	wcEx.lpszClassName   = &className$

	SetLastError (0) ' clear the last-error code
	IFZ RegisterClassExA (&wcEx) THEN RETURN $$TRUE ' error

	RETURN $$FALSE		'OK!

END FUNCTION

FUNCTION WndProc (hwnd, msg, wParam, lParam)		' callback function for window #winMain

	SHARED hInst
	SHARED hFontDefault		' default control font

	RECT rect

	SELECT CASE msg
		CASE $$WM_CREATE
			GOSUB winMain_Create
			SendMessageA (#radAW_CENTER, $$BM_SETCHECK, $$BST_CHECKED, 0)
			'
		CASE $$WM_CLOSE : DestroyWindow (#winMain)
		CASE $$WM_DESTROY
			#winMain = 0
			PostQuitMessage ($$WM_QUIT)
		CASE $$WM_COMMAND
			id         = LOWORD(wParam)
			notifyCode = HIWORD(wParam)
			hCtr       = lParam
			SELECT CASE id
				CASE $$btnOK
					IF notifyCode = $$BN_CLICKED THEN
						' create window #winAnimated
						SELECT CASE TRUE
							CASE $$BST_CHECKED == SendMessageA (#radHOR_POSITIVE, $$BM_GETCHECK, 0, 0)
								titleBar$ = "animate the window from left to right"
								'
							CASE $$BST_CHECKED == SendMessageA (#radHOR_NEGATIVE, $$BM_GETCHECK, 0, 0)
								titleBar$ = "animate the window from right to left"
								'
							CASE $$BST_CHECKED == SendMessageA (#radVER_POSITIVE, $$BM_GETCHECK, 0, 0)
								titleBar$ = "animate the window from top to bottom"
								'
							CASE $$BST_CHECKED == SendMessageA (#radVER_NEGATIVE, $$BM_GETCHECK, 0, 0)
								titleBar$ = "animate the window from bottom to top"
								'
							CASE $$BST_CHECKED == SendMessageA (#radAW_CENTER, $$BM_GETCHECK, 0, 0)
								titleBar$ = "expand the window outward"
								'
						END SELECT
						'
						' ***************** Begin window #winAnimated Generation *****************
						' register the window class for window #winAnimated
						#winAnimated_class$ = "winAnimatedClass"
						addrWndProc = &winAnimated_CallBack ()
						icon$ = ""
						menu$ = ""
						SetLastError (0)		' clear the last-error code
						bErr = RegisterWinClass (#winAnimated_class$, addrWndProc, icon$, menu$)		' register the window class
						IF bErr THEN		' error
							msg$ = "RegisterWinClass: Can't register the window class"
							GuiTellApiError (msg$)
						ELSE
							style = $$WS_OVERLAPPEDWINDOW
							#winAnimated = NewWindow (#winAnimated_class$, titleBar$, style, 80, 60, 400, 300, 0)		' create window #winAnimated
							IFZ #winAnimated THEN		' error
								msg$ = "NewWindow: Can't create window #winAnimated"
								XstAlert (msg$)
							ELSE
								ShowWindow (#winMain, $$SW_HIDE)		' hide the window #winMain
								XstCenterWindow (#winAnimated)		' auto-center the window #winAnimated
								ShowWindow (#winAnimated, $$SW_SHOWNORMAL)		' show the window #winAnimated
							END IF
						END IF
						' ***************** End window #winAnimated Generation *****************
					END IF
					'
				CASE $$btnExit : IF notifyCode = $$BN_CLICKED THEN DestroyWindow (#winMain)
				CASE ELSE : RETURN DefWindowProcA (hwnd, msg, wParam, lParam)
			END SELECT
		CASE ELSE : RETURN DefWindowProcA (hwnd, msg, wParam, lParam)
	END SELECT

	RETURN 0		'OK!

SUB winMain_Create

	#winMain = hwnd		' set the global handle

	' *************** Begin Controls Configuration **************
	#grpHorizontal = NewChild ($$BUTTON, "Horizontal Effect", $$BS_GROUPBOX, 48, 42, 130, 120, #winMain, $$grpHorizontal, 0)		' create button group #grpHorizontal
	IFZ #grpHorizontal THEN		' error
		msg$ = "NewChild: Can't create button group #grpHorizontal"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	' apply the fonts to the controls
	GuiSetFont (#grpHorizontal, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#grpVertical = NewChild ($$BUTTON, "Vertical Effect", $$BS_GROUPBOX, 196, 42, 130, 120, #winMain, $$grpVertical, 0)		' create button group #grpVertical
	IFZ #grpVertical THEN		' error
		msg$ = "NewChild: Can't create button group #grpVertical"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#grpVertical, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	style = $$WS_GROUP | $$WS_TABSTOP | $$BS_AUTORADIOBUTTON
	#radHOR_POSITIVE = NewChild ($$BUTTON, "left to right", style, 62, 80, 106, 20, #winMain, $$radHOR_POSITIVE, 0)		' create radio button #radHOR_POSITIVE
	IFZ #radHOR_POSITIVE THEN		' error
		msg$ = "NewChild: Can't create radio button #radHOR_POSITIVE"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	' apply the fonts to the controls
	GuiSetFont (#radHOR_POSITIVE, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#radHOR_NEGATIVE = NewChild ($$BUTTON, "right to left", $$BS_AUTORADIOBUTTON, 62, 130, 106, 20, #winMain, $$radHOR_NEGATIVE, 0)		' create radio button #radHOR_NEGATIVE
	IFZ #radHOR_NEGATIVE THEN		' error
		msg$ = "NewChild: Can't create radio button #radHOR_NEGATIVE"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#radHOR_NEGATIVE, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#radVER_POSITIVE = NewChild ($$BUTTON, "top to bottom", $$BS_AUTORADIOBUTTON, 212, 80, 106, 20, #winMain, $$radVER_POSITIVE, 0)		' create radio button #radVER_POSITIVE
	IFZ #radVER_POSITIVE THEN		' error
		msg$ = "NewChild: Can't create radio button #radVER_POSITIVE"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#radVER_POSITIVE, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#radVER_NEGATIVE = NewChild ($$BUTTON, "bottom to top", $$BS_AUTORADIOBUTTON, 212, 130, 106, 20, #winMain, $$radVER_NEGATIVE, 0)		' create radio button #radVER_NEGATIVE
	IFZ #radVER_NEGATIVE THEN		' error
		msg$ = "NewChild: Can't create radio button #radVER_NEGATIVE"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#radVER_NEGATIVE, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#radAW_CENTER = NewChild ($$BUTTON, "Center", $$BS_AUTORADIOBUTTON, 180, 176, 106, 20, #winMain, $$radAW_CENTER, 0)		' create radio button #radAW_CENTER
	IFZ #radAW_CENTER THEN		' error
		msg$ = "NewChild: Can't create radio button #radAW_CENTER"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#radAW_CENTER, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#btnOK = NewChild ($$BUTTON, "&OK", $$WS_TABSTOP, 74, 252, 76, 24, #winMain, $$btnOK, 0)		' create button #btnOK
	IFZ #btnOK THEN		' error
		msg$ = "NewChild: Can't create button #btnOK"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#btnOK, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	#btnExit = NewChild ($$BUTTON, "E&xit", $$WS_TABSTOP, 218, 252, 76, 24, #winMain, $$btnExit, 0)		' create button #btnExit
	IFZ #btnExit THEN		' error
		msg$ = "NewChild: Can't create button #btnExit"
		XstAlert (msg$)
		RETURN $$TRUE		' error
	END IF
	GuiSetFont (#btnExit, hFontDefault)		' apply the control default font
	' -------------------------------------------------------------------------------

	' **************** End Controls Configuration ***************

END SUB		' winMain_Create

END FUNCTION

FUNCTION winAnimated_CallBack (hwnd, msg, wParam, lParam) ' callback function for window #winAnimated

	SHARED hInst
	SHARED hFontDefault ' default control font

	RECT rect

	SELECT CASE msg
		CASE $$WM_CREATE : GOSUB winAnimated_Create
		CASE $$WM_SHOWWINDOW
			flags = 0
			SELECT CASE TRUE
				CASE $$BST_CHECKED == SendMessageA (#radHOR_POSITIVE, $$BM_GETCHECK, 0, 0)
					flags = $$AW_HOR_POSITIVE | $$AW_SLIDE | $$AW_ACTIVATE
					'
				CASE $$BST_CHECKED == SendMessageA (#radHOR_NEGATIVE, $$BM_GETCHECK, 0, 0)
					flags = $$AW_HOR_NEGATIVE | $$AW_SLIDE | $$AW_ACTIVATE
					'
				CASE $$BST_CHECKED == SendMessageA (#radVER_POSITIVE, $$BM_GETCHECK, 0, 0)
					flags = $$AW_VER_POSITIVE | $$AW_SLIDE | $$AW_ACTIVATE
					'
				CASE $$BST_CHECKED == SendMessageA (#radVER_NEGATIVE, $$BM_GETCHECK, 0, 0)
					flags = $$AW_VER_NEGATIVE | $$AW_SLIDE | $$AW_ACTIVATE
					'
				CASE $$BST_CHECKED == SendMessageA (#radAW_CENTER, $$BM_GETCHECK, 0, 0)
					flags = $$AW_CENTER | $$AW_ACTIVATE' exploding opening window
					'
			END SELECT
			IF flags THEN
				IFZ wParam THEN
					bShow = $$FALSE
				ELSE
					bShow = $$TRUE
				END IF
				fnStatus = lParam
				'
				' IF bShow && (fnStatus = 0) THEN
				' showing
				IF bShow THEN
					' showing
					IFZ fnStatus THEN
						' use slide animation and activate the window
						DIM args[2]
						args[0] = hwnd
						args[1] = 500		' milliseconds
						args[2] = flags
						ret = XstCall ("AnimateWindow", "user32", @args[])
						IFZ ret THEN
							' error
							msg$ = "Can't call AnimateWindow"
							GuiTellApiError (msg$)
						END IF
					END IF
				END IF
			END IF
			ShowWindow (#winAnimated_btnClose, $$SW_SHOWNORMAL) ' show the button "Close"
			'
		CASE $$WM_CLOSE ' closed by User
			GOSUB DestroyWindow_winAnimated
			ShowWindow (#winMain, $$SW_SHOWNORMAL) ' show the window #winMain
			'
		CASE $$WM_COMMAND
			id         = LOWORD(wParam)
			notifyCode = HIWORD(wParam)
			hCtr       = lParam
			SELECT CASE id
				CASE $$winAnimated_btnClose
					IF notifyCode = $$BN_CLICKED THEN
						GOSUB DestroyWindow_winAnimated
						ShowWindow (#winMain, $$SW_SHOWNORMAL) ' show the window #winMain
					END IF
					'
				CASE ELSE : RETURN DefWindowProcA (hwnd, msg, wParam, lParam)
			END SELECT
		CASE ELSE : RETURN DefWindowProcA (hwnd, msg, wParam, lParam)
	END SELECT

	RETURN 0 'OK!

SUB winAnimated_Create

	#winAnimated = hwnd ' set the global handle

	' *************** Begin Controls Configuration **************
	#winAnimated_btnClose = NewChild ($$BUTTON, "&Close", $$WS_TABSTOP, 146, 134, 76, 24, #winAnimated, $$winAnimated_btnClose, 0) ' create button #winAnimated_btnClose
	IFZ #winAnimated_btnClose THEN ' error
		msg$ = "NewChild: Can't create button #winAnimated_btnClose"
		XstAlert (msg$)
		RETURN $$TRUE ' error
	END IF
	' apply the fonts to the controls
	GuiSetFont (#winAnimated_btnClose, hFontDefault) ' apply the control default font
	ShowWindow (#winAnimated_btnClose, $$SW_HIDE) ' hide the button "Close"
	' -------------------------------------------------------------------------------
	' **************** End Controls Configuration ***************

END SUB ' winAnimated_Create

SUB DestroyWindow_winAnimated

	ShowWindow (#winAnimated_btnClose, $$SW_HIDE) ' hide the button "Close"
	flags = 0
	SELECT CASE TRUE
		CASE $$BST_CHECKED == SendMessageA (#radHOR_POSITIVE, $$BM_GETCHECK, 0, 0)
			flags = $$AW_HOR_POSITIVE | $$AW_SLIDE | $$AW_HIDE
			'
		CASE $$BST_CHECKED == SendMessageA (#radHOR_NEGATIVE, $$BM_GETCHECK, 0, 0)
			flags = $$AW_HOR_NEGATIVE | $$AW_SLIDE | $$AW_HIDE
			'
		CASE $$BST_CHECKED == SendMessageA (#radVER_POSITIVE, $$BM_GETCHECK, 0, 0)
			flags = $$AW_VER_POSITIVE | $$AW_SLIDE | $$AW_HIDE
			'
		CASE $$BST_CHECKED == SendMessageA (#radVER_NEGATIVE, $$BM_GETCHECK, 0, 0)
			flags = $$AW_VER_NEGATIVE | $$AW_SLIDE | $$AW_HIDE
			'
		CASE $$BST_CHECKED == SendMessageA (#radAW_CENTER, $$BM_GETCHECK, 0, 0)
			flags = $$AW_CENTER | $$AW_HIDE
			'
	END SELECT
	IF flags THEN
		DIM args[2]
		args[0] = hwnd
		args[1] = 500 ' milliseconds
		args[2] = flags
		ret = XstCall ("AnimateWindow", "user32", @args[])
		IFZ ret THEN		' error
			msg$ = "Can't call AnimateWindow"
			GuiTellApiError (msg$)
		END IF
	END IF
	'
	DestroyWindow (#winAnimated) ' destroy the window
	#winAnimated = 0
	UnregisterClassA (&#winAnimated_class$, hInst) ' unregister the window class

END SUB

END FUNCTION

END PROGRAM
